/*
	Mouse manager definitions.

	Created by Adrian Robinson
*/

#ifndef MOUSEMAN_H_INCLUDED
#define MOUSEMAN_H_INCLUDED

#include <sys/types.h>
#include <sys/mouse.h>

#define MOUSE_MAN_NAME		"/multi/Mouse"

#define MAX_EVENTS			25

enum _mm_event_types {
	_EVENT = 1,
	_NEWCLIENT,
	_DELCLIENT
};


/*
	Mouse manager event.
*/
typedef struct {
	enum _mm_event_types	type;
	nid_t					nid;
	struct mouse_event		event;
} MMEvent;


/*
	Message codes accepted by mouse manager.
*/
enum _mouse_man_code {
	_MOUSEMAN_QUERY = 1,
	_MOUSEMAN_OPEN,
	_MOUSEMAN_READ,
	_MOUSEMAN_CLOSE,
	_MOUSEMAN_STATS
};


/*
	Messages accepted by mouse manager.
*/

struct _mouseman_open {
	msg_t		type;
	pid_t		eproxy;
	nid_t		node;
};

struct _mouseman_read {
	msg_t		type;
	int			handle;
};

struct _mouseman_close {
	msg_t		type;
	int			handle;
};




/*
	Replies to accepted messages.
*/
struct _mouseman_query_reply {
	msg_t		status;
	nid_t		nid;
};

struct _mouseman_open_reply {
	msg_t		status;
	int			handle;
};

struct _mouseman_read_reply {
	msg_t		status;
	int			count;
	int			empty;
	MMEvent		events[MAX_EVENTS];
};

struct _mouseman_stats_reply {
	msg_t		status;
	void far	*ptr;
};


/* -------- library function prototype interface to mouse manager ------- */

int multi_mouse_open(char *name, pid_t *eproxy);
int multi_mouse_read(MMEvent **buffer, int *empty);
int multi_mouse_close(void);
void far *multi_mouse_stats(pid_t *mmpid);

#endif
