/*
	Mouse manager library functions.

	Created by Adrian Robinson
*/
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <i86.h>

#include <sys/proxy.h>
#include <sys/kernel.h>
#include <sys/name.h>
#include <sys/seginfo.h>

#include "mm_msg.h"


static pid_t pid;
static struct _mouseman_read_reply read_rply;
static int handle;

int multi_mouse_open(char *name, pid_t *eproxy)
{
	struct _mouseman_open			openMsg;
	struct _mouseman_open_reply		openRply;
	msg_t							queryMsg;
	struct _mouseman_query_reply	queryRply;
	pid_t veproxy;

	if (!name)
		name = MOUSE_MAN_NAME;

	if ((pid = qnx_name_locate(0, name, 0, NULL)) == -1)
		return -1;

	queryMsg = _MOUSEMAN_QUERY;

	if (Send(pid, &queryMsg, &queryRply, sizeof(queryMsg), sizeof(queryRply)))
		return -1;

	veproxy = qnx_proxy_attach(0, NULL, 0, -1);
	if (eproxy)
		*eproxy = veproxy;

	if (queryRply.nid != getnid()) 
		veproxy  = qnx_proxy_rem_attach(queryRply.nid, veproxy);

	openMsg.type   = _MOUSEMAN_OPEN;
	openMsg.eproxy = veproxy;
	openMsg.node   = getnid();

	if (Send(pid, &openMsg, &openRply, sizeof(openMsg), sizeof(openRply)))
		return -1;

	if (openRply.status) {
		errno = openRply.status;
		return -1;
	}

	handle = openRply.handle;
	return 0;
}



int   multi_mouse_read(MMEvent **buffer, int *empty)
{
	struct _mouseman_read	msg;

	if (buffer == NULL) {
		errno = EINVAL;
		return -1;
	}

	msg.type        = _MOUSEMAN_READ;
	msg.handle = handle;

	if (Send(pid, &msg, &read_rply, sizeof(msg), sizeof(read_rply)))
		return -1;

	*buffer = &read_rply.events;
	if (empty)
		*empty = read_rply.empty;

	if (read_rply.status) {
		errno = read_rply.status;
		return -1;
	}


	return read_rply.count;
}



int   multi_mouse_close(void)
{
	struct _mouseman_close msg;
	msg_t rply;

	msg.type         = _MOUSEMAN_CLOSE;
	msg.handle = handle;

	if (Send(pid, &msg, &rply, sizeof(msg), sizeof(rply)))
		return -1;

	if (rply) {
		errno = rply;
		return -1;
	}

	return 0;
}



void far *multi_mouse_stats(pid_t *mmpid)
{
	msg_t							queryMsg;
	struct _mouseman_query_reply	queryRply;
	msg_t							statsMsg;
	struct _mouseman_stats_reply	statsRply;
	
	if ((pid = qnx_name_locate(0, MOUSE_MAN_NAME, 0, NULL)) == -1)
		return NULL;

	queryMsg = _MOUSEMAN_QUERY;

	if (Send(pid, &queryMsg, &queryRply, sizeof(queryMsg), sizeof(queryRply)))
		return NULL;

	if (queryRply.nid != getnid()) {
		errno = ENOREMOTE;
		return NULL;
	}

	statsMsg = _MOUSEMAN_STATS;

	if (Send(pid, &statsMsg, &statsRply, sizeof(statsMsg), sizeof(statsRply)))
		return NULL;

	if (mmpid)
		*mmpid = pid;

	return MK_FP(qnx_segment_get(pid, FP_SEG(statsRply.ptr), _PMF_DATA_R), 0);
}
