/*
	Simple program to test mouse manager.

	Created by Adrian Robinson
*/
#include <stdio.h>
#include <stdlib.h>

#include <sys/proxy.h>
#include <sys/kernel.h>

#include "mm_msg.h"


static void dump_events(MMEvent *events, int count)
{
	register int i;

	for (i = 0; i < count; i++, events++) {
		switch (events->type) {
		case _EVENT:
			printf("Mouse event from node %ld\n", events->nid);
			break;

		case _NEWCLIENT:
			printf("New Client on node %ld\n", events->nid);
			break;

		case _DELCLIENT:
			printf("Client gone on node %ld\n", events->nid);
			break;
		}
	}
}


void main(void)
{
	msg_t msg;
	pid_t pid, eproxy;
	MMEvent *events;

	if (multi_mouse_open(NULL, &eproxy)) {
		perror("Can't multi_mouse_open");
		printf("Make sure MultiMouse is already running in background.\n");
		printf("Make sure this node has the Mouse server running.\n");
		exit(EXIT_FAILURE);
	}

	printf("Move the mouse to receive events.\n");
	printf("Run this program on other nodes with mice attached or\n");
	printf("on other consoles on this node.\n");
	while (1) {
		pid = Receive(0, &msg, sizeof(msg));

		if (pid == eproxy) {
			int empty, count;

			do {
				if ((count = multi_mouse_read(&events, &empty)) == -1)
					perror("multi_mouse_read");
	
				dump_events(events, count);
			} while (!empty);
		}
		else {
			msg_t status = 0;

			printf("Unknown message %u\n", msg);

			Reply(pid, &status, sizeof(status));
		}
	}
}
