/*
	Simple monitor program to view mouse manager internals.

	Created by Adrian Robinson
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <conio.h>

#include <sys/timers.h>
#include <sys/qnxterm.h>
#include <sys/psinfo.h>

#include "local.h"


static void showMMStats(MMStats far *stats)
{
	register int count;
	register MouseClient far *client = stats->clients;

	term_printf(0, 26, TERM_YELLOW, "< Mouse Manager Statistics >");

	term_printf(2,  2, TERM_CYAN,  "Max Clients");
	term_printf(2, 18, TERM_GREEN, "%5d", stats->maxClients);

	term_printf(2, 40, TERM_CYAN,  "Client Count");
	term_printf(2, 58, TERM_GREEN, "%5d", stats->clientCount);

	term_printf(4,  2, TERM_CYAN,  "Client #   Node     Pid   Unread Events");

	for (count = 0; count < stats->maxClients; count++, client++) {
		term_printf(5+count, 4, TERM_CYAN,  "%2d)", count+1);

		if (client->pid)
			term_printf(5+count, 12, TERM_GREEN, "%5ld   %5u     %5u",
				client->nid, client->pid, client->eventCount);
		else
			term_clear(TERM_CLS_EOL);
	}
}


void main(void)
{
	timer_t time_id;
	struct itimercb timecb;
	struct itimerspec timespec;
	MMStats far *mmStats;
	pid_t mmPid;
	int done = 0, term = 0;
	struct _psinfo psinfo;
	int status;

	if ((mmStats = multi_mouse_stats(&mmPid)) == NULL) {
		fprintf(stderr, "Can't get mouse manager status segment: %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}

	if ((time_id = mktimer(TIMEOFDAY, _TNOTIFY_SLEEP, &timecb)) == -1) {
		term_restore();
		perror("Can't mktimer");
		exit(EXIT_FAILURE);
	}

	timespec.it_value.tv_sec     = 0L;
	timespec.it_value.tv_nsec    = 100000000L;
	timespec.it_interval.tv_sec  = 0L;
	timespec.it_interval.tv_nsec = 100000000L;

	if (term_load() == -1) {
		perror("Can't load terminal definition.\n");
		exit(EXIT_FAILURE);
	}

	term_clear(TERM_CLS_SCRH);
	while (!done) {
		showMMStats(mmStats);

		status = qnx_psinfo(0, mmPid, &psinfo, 0, NULL);
		if (status == -1 || psinfo.pid != mmPid) {
			term_clear(TERM_CLS_SCRH);
			term = done = 1;
		}

		if (kbhit()) {
			int ch;

			if ((ch = getch()) == 0xFF)
				ch = getch();

			if (ch == K_ESC || ch == K_F10) {
				term_clear(TERM_CLS_SCRH);
				done = 1;
			}
		}

		term_flush();

		if (!done)
			reltimer(time_id, &timespec, NULL);
	}

	term_restore();
	if (term)
		printf("Mouse manager task has terminated.\n");
}
