/*
	Mouse manager.

	Created by Adrian Robinson
*/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <i86.h>

#include <sys/name.h>
#include <sys/seginfo.h>
#include <sys/kernel.h>
#include <sys/psinfo.h>

#include "local.h"

#define	PFLAGS	(_PPF_SERVER|_PPF_PRIORITY_REC|_PPF_SIGCATCH)


MMStats far *eMMStats;

static MouseManMsg   sMsg;
static MouseManReply sRply;
static char *sMyName = MOUSE_MAN_NAME;

static struct _sysmsg_version_reply sVersionStamp = {
	"MultiMouse",
	__DATE__,
	100,
	' ',
	0
};


static void sysMsg(msg_t subtype, pid_t pid)
{
	struct _sysmsg_hdr_reply rply;
	struct _mxfer_entry mx_entry[2];

	rply.status = EOK;

	switch(subtype) {
	case _SYSMSG_SUBTYPE_DEATH:
	case _SYSMSG_SUBTYPE_SIGNAL:
		delClient(pid, -1);
		Reply(pid, &rply, sizeof(rply));
		break;

	case _SYSMSG_SUBTYPE_VERSION:
		_setmx(&mx_entry[0], &rply, sizeof(rply));
		_setmx(&mx_entry[1], &sVersionStamp, sizeof(sVersionStamp));
		Replymx(pid, 2, &mx_entry);
		break;

	case _SYSMSG_SUBTYPE_TRACE:
	default:
		rply.status = ENOSYS;
		Reply(pid, &rply, sizeof(rply));
		break;
	}
}


static int init(int argc, char *argv[])
{
	int	c, err = 0;
	int maxClients = MAX_CLIENTS;
	int size;

	while((c = getopt(argc, argv, "N:c:")) != -1) {
		switch(c) {
		case 'N':
			sMyName = optarg;
			break;

		case 'c':
			maxClients = atoi(optarg);
			break;
		default:
			err++;
			break;
		}
	}

	size = sizeof(*eMMStats) + (maxClients - 1) * sizeof(MouseClient);
	eMMStats = MK_FP(qnx_segment_alloc(size), 0);
	eMMStats->maxClients = maxClients;

	return err;
}


void main(int argc, char *argv[])
{
	pid_t pid;
	size_t size;
	int handle;

	if (init(argc, argv)) {
		print_usage(argv);
		exit(EXIT_FAILURE);
	}

	if (qnx_name_attach(getnid(), sMyName) == -1) {
		perror("Can't qnx_name_attach");
		exit(EXIT_FAILURE);
	}

	if (qnx_pflags(PFLAGS, PFLAGS, NULL, NULL)) {
		perror("Can't set qnx_pflags");
		exit(EXIT_FAILURE);
	}

	while (1) {
		pid = Receive(0, &sMsg, sizeof(sMsg));

		if ((handle = findClient(pid, 0, 1)) != -1)
			addEvent(handle);
		else {
			sRply.status = EOK;
			size = sizeof(sRply.status);

			switch (sMsg.type) {
			case _SYSMSG:
				sysMsg(sMsg.hdr.subtype, pid);
				continue;

			case _MOUSEMAN_QUERY:
				sRply.query.nid = getnid();
				size = sizeof(sRply.query);
				break;

			case _MOUSEMAN_OPEN:
				sRply.status = addClient(pid, &sMsg.open, &sRply.open.handle);
				size = sizeof(sRply.open);
				break;

			case _MOUSEMAN_READ:
				sRply.status = readEvent(pid, sMsg.read.handle, &sRply.read, &size);
				break;

			case _MOUSEMAN_CLOSE:
				sRply.status = delClient(pid, sMsg.close.handle);
				break;

			case _MOUSEMAN_STATS:
				sRply.stats.ptr = eMMStats;
				qnx_segment_arm(pid, FP_SEG(eMMStats), _PMF_DATA_R);
				size = sizeof(sRply.stats);
				break;

			default:
				sRply.status = ENOSYS;
				break;
			}

			Reply(pid, &sRply, size);
		}
	}
}
